/*
 *  Camera.cpp
 *  metricspace
 *
 *  Created by Michael Tauraso on 12/17/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include <cmath>
#if defined(_WIN32)
#	include <GL/glut.h>
#else
#	include <GLUT/glut.h>
#endif

#define PI 3.14159265358

#include "Camera.h"

Camera* Camera::m_instance = 0;

Camera* Camera::getInstance()
{
	if(!m_instance)
		m_instance = new Camera();
	
	return m_instance;
}

Camera::Camera()
:m_lookAtPos(0,0,0), m_up(0,0,1), 
 m_neardist(1), m_fardist(1000),
 m_aspect(1), m_fov(45)
{
	setCartLocation(Tuple(10,10,10));
	glutReshapeFunc(Reshape);
}

void Camera::setUpVec(Tuple up)
{
	m_up = up;
}

Tuple Camera::getUpVec()
{
	return m_up;
}

void Camera::setLookAt(Tuple pos)
{
	m_lookAtPos = pos;
}
Tuple Camera::getLookAt()
{
	return m_lookAtPos;
}

void Camera::setAspect(float ratio)
{
	m_aspect = ratio;
	//possibly unnecessary?
	updateFrust();
}

void Camera::setFOV(float deg)
{
	m_fov = deg;
		//possibly unnecessary?
	updateFrust();
}

void Camera::setCartLocation(Tuple pos)
{
	m_cartPos = pos;
	//all the math.h functions return in rads
	//we use degrees
	//rho
	m_spherePos[0] = sqrt(pos.dot(pos));
	//theta
	m_spherePos[1] = atan(pos[1]/pos[0])*180/PI;
	//phi
	m_spherePos[2] = acos(pos[2]/sqrt(pos.dot(pos)))*180/PI;
}

void Camera::setSphereLocation(Tuple pos)
{
	//all the math.h functions use radians
	//we have degrees
	m_spherePos = pos;
	m_cartPos[0] = pos[0] * sin(pos[2]*PI/180) * cos(pos[1]*PI/180); 
	m_cartPos[1] = pos[0] * sin(pos[2]*PI/180) * sin(pos[1]*PI/180);
	m_cartPos[2] = pos[0] * cos(pos[2]*PI/180);
	
}

Tuple Camera::getAbsolutePos()
{
	return m_cartPos + m_lookAtPos;
}

Tuple Camera::getCartLocation()
{
	return m_cartPos;
}

Tuple Camera::getSphereLocation()
{
	return m_spherePos;
}

void Camera::Look()
{
	//glMatrixMode(GL_MODELVIEW);
	//glLoadIdentity();
	
	Tuple eyePos = getAbsolutePos();
	gluLookAt(eyePos[0], eyePos[1], eyePos[2], 
		      m_lookAtPos[0], m_lookAtPos[1], m_lookAtPos[2],
			  m_up[0], m_up[1], m_up[2]);
}
void Camera::updateFrust()
{
	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	// Set the correct perspective.
	gluPerspective(m_fov,m_aspect,m_neardist,m_fardist);
	glMatrixMode(GL_MODELVIEW);
}

void Reshape(int w, int h)
{
	Camera * camera = Camera::getInstance();
	
	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if(h == 0)
		h = 1;
	
	camera->m_aspect = 1.0* w / h;
	
	// Set the viewport to be the entire window
	glViewport(0, 0, w, h);
	
	camera->updateFrust();
	//camera->updatePos();
}
