/*
 *  FileParser.cpp
 *  metricspace
 *
 *  Created by Michael Tauraso on 12/18/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include <fstream>
#include <sstream>
#include <stdexcept>

#include "FileParser.h"


const char** FileParser::parseLines(std::string filename, size_t&size)
{
	std::vector<std::string> fileData = parseLines(filename);
	
	// allocate and change vector<string> into
	// const char ** with each line null terminated
	const char ** returnme = new const char * [ fileData.size() ];
	
	for(size_t i = 0; i < fileData.size(); i++)
	{
		char * current = new char[fileData[i].size()+1];
		strcpy(current, fileData[i].c_str());
		returnme[i] = current;
	}
	size = fileData.size();
	return returnme;
}

std::vector<std::string> FileParser::parseLines(std::string filename)
{
	//open the file
	std::vector<std::string> fileData;
	std::ifstream file;
	file.open(filename.c_str(), std::fstream::in);
	if(!file.is_open())
	{
		throw std::runtime_error("File not found: " + filename);
		return fileData;
	}
	
	// count the newlines
	// and find the length of the longest line
	// while reading into nice c++ vector< string>

	while(!file.eof())
	{
		std::string current;
		getline(file, current);
		fileData.push_back(std::string(current));
	}
	
	file.close();
	return fileData;
}

void FileParser::freeLines(const char** lines, size_t size)
{
	for(size_t i=0; i< size; i++)
		delete [] lines[i];
	delete[] lines;
}

std::vector< std::vector <FileToken> > FileParser::tokenize(std::string filename)
{
	std::vector<std::string> fileData = parseLines(filename);
	std::vector< std::vector <FileToken> > returnme;
	for(std::vector<std::string>::iterator i = fileData.begin(); 
		i!=fileData.end(); i++)
	{
		returnme.push_back(tokenizeLine(*i));
	}
	return returnme;
}

std::vector<FileToken> FileParser::tokenizeLine(std::string line)
{
	std::stringstream linestream(line);
	
	std::vector<FileToken> returnme;
	while(linestream.good())
	{
		std::string current;
		linestream >> current;
		returnme.push_back(FileToken(current));
	}
	
	return returnme;
}

FileToken::FileToken(std::string input)
:m_data(input)
{
}

std::string FileToken::getString()
{
	return m_data;
}

int FileToken::getInt()
{
	return atoi(m_data.c_str());
}

float FileToken::getFloat()
{
	return atof(m_data.c_str());
}