/*
 *  Matrix4.cpp
 *  metricspace
 *
 *  Created by Michael Tauraso on 12/17/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include <stdexcept>
#include <vector>
#include "FileParser.h"

#include "Matrix.h"

Matrix::Matrix()
{
	m_rows[0] = Tuple(0,0,0,0);
	m_rows[1] = Tuple(0,0,0,0);
	m_rows[2] = Tuple(0,0,0,0);
	m_rows[3] = Tuple(0,0,0,0);
}

Matrix::Matrix(Tuple r1,Tuple r2,Tuple r3,Tuple r4)
{
	m_rows[0] = r1;
	m_rows[1] = r2;
	m_rows[2] = r3;
	m_rows[3] = r4;
}

#define min(a,b) (((a)<(b))?(a):(b))

Matrix::Matrix(std::string filename)
{
	std::vector<std::vector<FileToken> > data = FileParser::tokenize(filename);
	
	for(int i=0; i< min(data.size(),4); i++)
	{
		float t[4] = {0,0,0,0};
		for(int j=0; j< min(data[i].size(),4); j++)
		{
			t[j] = data[i][j].getFloat();
		}
		m_rows[i] = Tuple(t[0],t[1],t[2],t[3]);
	}
}

Matrix::Matrix(const Matrix& rhs)
{
	for(int i=0; i<4 ; i++)
	{
		m_rows[i] = rhs.m_rows[i];
	}
}

GLfloat * Matrix::getMat3()
{
	return getMatn(3);
}

GLfloat * Matrix::getMatn(int n)
{
	if(n < 2 || n > 4)
	{
		return NULL;
	}
	
	GLfloat * returnme = new GLfloat [n*n];
	for(int i=0; i<n; i++)
	{
		int offset = n*i;
		for(int j=0; j<n; j++)
		{
			returnme[offset+j] = m_rows[i][j];
		}
	}
	return returnme;
}

Tuple& Matrix::operator [] (int i)
{
	if(i < 0 || i >= 4)
	{
		throw std::range_error("Matrix array index out of bounds");
	}
	return m_rows[i];
}

const Tuple & Matrix::operator[] (int i) const
{
	if(i < 0 || i >= 4)
	{
		throw std::range_error("Matrix array index out of bounds");
	}
	return m_rows[i];
}

ColumnTuple operator * (const Matrix & lhs, const ColumnTuple & rhs)
{
	return ColumnTuple(Tuple(lhs[0].dot(rhs),lhs[1].dot(rhs),
							 lhs[2].dot(rhs),lhs[3].dot(rhs)) ); 
}

RowTuple operator * (const RowTuple & lhs, const Matrix& rhs)
{
	RowTuple returnme = Tuple (0,0,0,0);
	
	for(int i=0; i< 4; i++)
	{
		Tuple ithcol = Tuple(rhs.m_rows[0][i],
							 rhs.m_rows[1][i],
							 rhs.m_rows[2][i],
							 rhs.m_rows[3][i]);
		
		returnme[i] = ithcol.dot(lhs);
	}
	
	return returnme;
}