/*
 *  Metric.cpp
 *  metricspace
 *
 *  Created by Michael Tauraso on 12/17/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */
#include <cmath>
#include <stdexcept>

#include "Shader.h"
#include "FileParser.h"
#include "Metric.h"

//called by all metrics after their constructor
void Metric::Initialize()
{
	Shader * myShader = new Shader("metric.vert", GL_VERTEX_SHADER_ARB);
	m_mainProgram.attach(myShader);
	
	myShader = new Shader("metric.frag", GL_FRAGMENT_SHADER_ARB);
	m_mainProgram.attach(myShader);
	
	m_mainProgram.link();
	//m_mainProgram.validate();
}


void Metric::Apply()
{
	m_mainProgram.use();
	Use();
}



Tuple Metric::transform(const Tuple & rhs)
{
	Tuple returnme = rhs / EuclideanMetric().distance(rhs);
	
	returnme *= distance(rhs);
	
	return rhs;	
}

EuclideanMetric::EuclideanMetric()
{
	Shader * myShader = new Shader("euclid.vert", GL_VERTEX_SHADER_ARB);
	m_mainProgram.attach(myShader);
	Initialize();
}

float EuclideanMetric::distance(Tuple x, Tuple y)
{
	Tuple coord = x-y;
	return sqrt(coord.dot(coord));
}

Tuple EuclideanMetric::transform(const Tuple & rhs)
{
	return rhs;
}


CityBlockMetric::CityBlockMetric()
{
	
	Shader * myShader = new Shader("cityblock.vert", GL_VERTEX_SHADER_ARB);
	m_mainProgram.attach(myShader);
	Initialize();
}

float CityBlockMetric::distance(Tuple x, Tuple y)
{
	Tuple coord = x-y;
	return fabsf(coord[0] + coord[1] + coord[2] + coord[3]);
}


RiemannianMetric::RiemannianMetric(Matrix mat)
:m_mat(mat)
{
	Shader * myShader = new Shader("riemann.vert", GL_VERTEX_SHADER_ARB);
	m_mainProgram.attach(myShader);
	
	Initialize();
}

RiemannianMetric::RiemannianMetric(std::string filename)
:m_mat(filename)
{
	/*m_mat = Matrix(Tuple(1,1,0),
				   Tuple(1,2,0),
				   Tuple(0,0,1));*/
	
	Shader * myShader = new Shader("riemann.vert", GL_VERTEX_SHADER_ARB);
	m_mainProgram.attach(myShader);
	
	Initialize();	
}

void RiemannianMetric::Use()
{
	m_matHandle = glGetUniformLocationARB(m_mainProgram.getHandle(),"g_matrix");
	glUniformMatrix3fvARB(m_matHandle, 1, 1, m_mat.getMat3());
}

float RiemannianMetric::distance(Tuple x, Tuple y)
{	
	ColumnTuple coord = x-y;
	
	return sqrt(coord.transpose()*m_mat*coord);	
}
