/*
 *  Tuple.cpp
 *  metricspace
 *
 *  Created by Michael Tauraso on 12/17/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include<stdexcept>
#include<cmath>

#include "Tuple.h"
#include "Matrix.h"

Tuple::Tuple()
{
	for(int i=0; i<4; i++)
		m_val[i] = 0;
}

Tuple::Tuple(float x, float y, float z, float w)
{
	m_val[0] = x;
	m_val[1] = y;
	m_val[2] = z;
	m_val[3] = w;
}

Tuple::Tuple(const Tuple & rhs)
{
	for(int i=0; i< 4; i++)
		m_val[i] = rhs.m_val[i];
}

float& Tuple::operator [](const int i)
{
	if(i>=4 || i<0)
	{
		throw std::range_error("Tuple array index out of bounds");
	}
	return m_val[i];
}

const float& Tuple::operator [](const int i) const
{
	if(i>=4 || i<0)
	{
		throw std::range_error("Tuple array index out of bounds");
	}
	return m_val[i];
}

Tuple Tuple::operator +(const Tuple& rhs) const
{
	Tuple returnme;
	for(int i=0; i<4; i++)
	{
		returnme.m_val[i] = m_val[i] + rhs.m_val[i];
	}
	return returnme;
}

Tuple Tuple::operator -(const Tuple& rhs) const
{
	Tuple returnme;
	for(int i=0; i<4; i++)
	{
		returnme.m_val[i] = m_val[i] - rhs.m_val[i];
	}
	return returnme;
}

Tuple Tuple::operator *(const float rhs) const
{
	Tuple returnme;
	for(int i=0; i< 4; i++)
	{
		returnme.m_val[i] = m_val[i] * rhs;
	}
	return returnme;
}

Tuple Tuple::operator /(const float rhs) const
{
	Tuple returnme;
	for(int i=0; i< 4; i++)
	{
		returnme.m_val[i] = m_val[i] / rhs;
	}
	return returnme;
}

Tuple& Tuple::operator *= (const float rhs)
{
	*this = *this * rhs;
	return *this;
}

Tuple& Tuple::operator /= (const float rhs)
{
	*this = *this / rhs;
	return *this;
}

Tuple& Tuple::operator += (const Tuple & rhs)
{
	*this = *this + rhs;
	return *this;
}

Tuple& Tuple::operator -= (const Tuple & rhs)
{
	*this = *this - rhs;
	return *this;
}

float Tuple::dot(const Tuple& rhs) const
{
	float returnme = 0;
	
	for(int i=0; i<4; i++)
	{
		returnme += m_val[i]*rhs.m_val[i];
	}
	
	return returnme;
}

Matrix Tuple::outer(const Tuple& rhs) const
{
	Matrix returnme;
	
	for(int i=0; i<4; i++)
	{
		for(int j=0; j<4; j++)
		{
			returnme[i][j] = this->m_val[i] * rhs[j];
		}
	}
	
	return returnme;
}

Tuple Tuple::cross(const Tuple& rhs) const
{
	Tuple returnme = Tuple (0,0,0);
	//cross product has no meaning for forth component
	// so we pretend that they're just 3-vectors
	for(int i=0; i<3; i++)
	{
		returnme[i] = m_val[(i+1)%3]*rhs.m_val[(i+2)%3];
		returnme[i] -= m_val[(i+2)%3] * rhs.m_val[(i+1)%3];
	}
	return returnme;
}

Tuple Tuple::normalize() const
{
	return *this / sqrt(this->dot(*this));
}

//
//Row and column Tuples stuff
//


//column tuple
ColumnTuple::ColumnTuple(const Tuple & rhs)
{
	*this = Tuple(rhs);
}

RowTuple ColumnTuple::transpose() const 
{
	return ColumnTuple(*((Tuple *)this));
}


Matrix ColumnTuple::operator * (const RowTuple & rhs) const
{
	return this->outer(rhs);
}


//row tuple
RowTuple::RowTuple(const Tuple& rhs)
{
	*this = Tuple(rhs);
}

ColumnTuple RowTuple::transpose() const 
{
	return ColumnTuple(*((Tuple *)this));
}

float RowTuple::operator*( const ColumnTuple & rhs) const
{
	return this->dot(rhs);
}

